@@ -2,7 +2,7 @@
  *
  * common.c - Flickcurl common functions
  *
- * Copyright (C) 2007-2011, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2007-2012, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -63,9 +63,9 @@
 
 #include <libxml/xmlsave.h>
 
-const char* const flickcurl_short_copyright_string = "Copyright 2007-2011 David Beckett.";
+const char* const flickcurl_short_copyright_string = "Copyright 2007-2012 David Beckett.";
 
-const char* const flickcurl_copyright_string = "Copyright (C) 2007-2011 David Beckett - http://www.dajobe.org/";
+const char* const flickcurl_copyright_string = "Copyright (C) 2007-2012 David Beckett - http://www.dajobe.org/";
 
 const char* const flickcurl_license_string = "LGPL 2.1 or newer, GPL 2 or newer, Apache 2.0 or newer.\nSee http://librdf.org/flickcurl/ for full terms.";
 
@@ -204,28 +204,41 @@ flickcurl*
 flickcurl_new_with_handle(void* curl_handle)
 {
   flickcurl* fc;
-
+  size_t len;
+  
   fc = (flickcurl*)calloc(1, sizeof(flickcurl));
   if(!fc)
     return NULL;
 
-  fc->service_uri = strdup(flickcurl_flickr_service_uri);
-  fc->upload_service_uri = strdup(flickcurl_flickr_upload_service_uri);
-  fc->replace_service_uri = strdup(flickcurl_flickr_replace_service_uri);
-  fc->oauth_request_token_uri = strdup(flickcurl_flickr_oauth_request_token_uri);
-  fc->oauth_access_token_uri = strdup(flickcurl_flickr_oauth_access_token_uri);
+  len = strlen(flickcurl_flickr_service_uri);
+  fc->service_uri = (char*)malloc(len + 1);
+  memcpy(fc->service_uri, flickcurl_flickr_service_uri, len + 1);
+  
+  len = strlen(flickcurl_flickr_upload_service_uri);
+  fc->upload_service_uri = (char*)malloc(len + 1);
+  memcpy(fc->upload_service_uri, flickcurl_flickr_upload_service_uri, len +1);
+
+  len = strlen(flickcurl_flickr_replace_service_uri);
+  fc->replace_service_uri = (char*)malloc(len + 1);
+  memcpy(fc->replace_service_uri, flickcurl_flickr_replace_service_uri, len +1);
+
+  len = strlen(flickcurl_flickr_oauth_request_token_uri);
+  fc->oauth_request_token_uri = (char*)malloc(len + 1);
+  memcpy(fc->oauth_request_token_uri, flickcurl_flickr_oauth_request_token_uri, len +1);
+
+  len = strlen(flickcurl_flickr_oauth_access_token_uri);
+  fc->oauth_access_token_uri = (char*)malloc(len + 1);
+  memcpy(fc->oauth_access_token_uri, flickcurl_flickr_oauth_access_token_uri, len +1);
 
   /* DEFAULT delay between requests is 1000ms i.e 1 request/second max */
   fc->request_delay = 1000;
 
-#ifdef FLICKCURL_OAUTH
   fc->mt = mtwist_new();
   if(!fc->mt) {
     free(fc);
     return NULL;
   }
   mtwist_init(fc->mt, mtwist_seed_from_system(fc->mt));
-#endif
 
   fc->curl_handle = (CURL*)curl_handle;
   if(!fc->curl_handle) {
@@ -288,8 +301,6 @@ flickcurl_free(flickcurl *fc)
     xmlFreeParserCtxt(fc->xc); 
   }
 
-  if(fc->api_key)
-    free(fc->api_key);
   if(fc->secret)
     free(fc->secret);
   if(fc->auth_token)
@@ -360,12 +371,10 @@ flickcurl_free(flickcurl *fc)
   if(fc->uri)
     free(fc->uri);
 
-#ifdef FLICKCURL_OAUTH
   if(fc->mt)
     mtwist_free(fc->mt);
 
   flickcurl_oauth_free(&fc->od);
-#endif
 
   free(fc);
 }
@@ -448,10 +457,12 @@ flickcurl_set_tag_handler(flickcurl* fc,
 void
 flickcurl_set_user_agent(flickcurl* fc, const char *user_agent)
 {
-  char *ua_copy = (char*)malloc(strlen(user_agent)+1);
+  size_t len = strlen(user_agent);
+  char *ua_copy = (char*)malloc(len + 1);
   if(!ua_copy)
     return;
-  strcpy(ua_copy, user_agent);
+
+  memcpy(ua_copy, user_agent, len + 1);
   
   fc->user_agent = ua_copy;
 }
@@ -467,10 +478,12 @@ flickcurl_set_user_agent(flickcurl* fc, const char *user_agent)
 void
 flickcurl_set_proxy(flickcurl* fc, const char *proxy)
 {
-  char *proxy_copy = (char*)malloc(strlen(proxy)+1);
+  size_t len = strlen(proxy);
+  char *proxy_copy = (char*)malloc(len + 1);
   if(!proxy_copy)
     return;
-  strcpy(proxy_copy, proxy);
+
+  memcpy(proxy_copy, proxy, len + 1);
   
   fc->proxy = proxy_copy;
 }
@@ -487,21 +500,22 @@ void
 flickcurl_set_http_accept(flickcurl* fc, const char *value)
 {
   char *value_copy;
-  size_t len = 8; /* strlen("Accept:")+1 */
+  size_t len = 7; /* strlen("Accept:") */
   
   if(value)
-    len += 1+strlen(value); /* " "+value */
+    len += 1 + strlen(value); /* " "+value */
   
-  value_copy = (char*)malloc(len);
+  value_copy = (char*)malloc(len + 1);
   if(!value_copy)
     return;
+
   fc->http_accept = value_copy;
 
-  strcpy(value_copy, "Accept:");
+  memcpy(value_copy, "Accept:", 8); /* copy NUL */
   value_copy += 7;
   if(value) {
     *value_copy++ = ' ';
-    strcpy(value_copy, value);
+    memcpy(value_copy, value, (len - 8) + 1);
   }
 
 }
@@ -519,15 +533,20 @@ flickcurl_set_http_accept(flickcurl* fc, const char *value)
 void
 flickcurl_set_service_uri(flickcurl *fc, const char *uri)
 {
+  size_t len;
+  
   if(!uri)
     uri = flickcurl_flickr_service_uri;
     
 #if FLICKCURL_DEBUG > 1
-    fprintf(stderr, "Service URI set to: '%s'\n", uri);
+  fprintf(stderr, "Service URI set to: '%s'\n", uri);
 #endif
-    if(fc->service_uri)
-      free(fc->service_uri);
-    fc->service_uri = strdup(uri);
+  if(fc->service_uri)
+    free(fc->service_uri);
+
+  len = strlen(uri);
+  fc->service_uri = (char*)malloc(len + 1);
+  memcpy(fc->service_uri, uri, len + 1);
 }
 
 
@@ -544,6 +563,8 @@ flickcurl_set_service_uri(flickcurl *fc, const char *uri)
 void
 flickcurl_set_upload_service_uri(flickcurl *fc, const char *uri)
 {
+  size_t len;
+
   if(!uri)
     uri = flickcurl_flickr_upload_service_uri;
     
@@ -552,7 +573,10 @@ flickcurl_set_upload_service_uri(flickcurl *fc, const char *uri)
 #endif
     if(fc->upload_service_uri)
       free(fc->upload_service_uri);
-    fc->upload_service_uri = strdup(uri);
+
+  len = strlen(uri);
+  fc->upload_service_uri = (char*)malloc(len + 1);
+  memcpy(fc->upload_service_uri, uri, len + 1);
 }
 
 
@@ -569,6 +593,8 @@ flickcurl_set_upload_service_uri(flickcurl *fc, const char *uri)
 void
 flickcurl_set_replace_service_uri(flickcurl *fc, const char *uri)
 {
+  size_t len;
+
   if(!uri)
     uri = flickcurl_flickr_replace_service_uri;
     
@@ -577,7 +603,10 @@ flickcurl_set_replace_service_uri(flickcurl *fc, const char *uri)
 #endif
     if(fc->replace_service_uri)
       free(fc->replace_service_uri);
-    fc->replace_service_uri = strdup(uri);
+
+  len = strlen(uri);
+  fc->replace_service_uri = (char*)malloc(len + 1);
+  memcpy(fc->replace_service_uri, uri, len + 1);
 }
 
 
@@ -586,17 +615,33 @@ flickcurl_set_replace_service_uri(flickcurl *fc, const char *uri)
  * @fc: flickcurl object
  * @api_key: API Key
  *
- * Set application API Key for flickcurl requests
+ * Set legacy Flickr auth application API Key (OAuth Client key)
+ *
+ * For OAuth this is not sufficient and
+ * flickcurl_set_oauth_client_key() and
+ * flickcurl_set_oauth_client_secret() be used
+ * to pass both the client key and client secret.
  */
 void
 flickcurl_set_api_key(flickcurl* fc, const char *api_key)
 {
+  size_t len;
+  
 #if FLICKCURL_DEBUG > 1
   fprintf(stderr, "API Key: '%s'\n", api_key);
 #endif
-  if(fc->api_key)
-    free(fc->api_key);
-  fc->api_key = strdup(api_key);
+  if(fc->od.client_key)
+    free(fc->od.client_key);
+
+  len = strlen(api_key);
+  fc->od.client_key = (char*)malloc(len + 1);
+  memcpy(fc->od.client_key, api_key, len + 1);
+  fc->od.client_key_len = len;
+
+  /* Mainly for flickcurl_auth_oauth_getAccessToken() to sign the call
+   * exchanging tokens 
+   */
+  fc->api_key = fc->od.client_key;
 }
 
 
@@ -604,7 +649,7 @@ flickcurl_set_api_key(flickcurl* fc, const char *api_key)
  * flickcurl_get_api_key:
  * @fc: flickcurl object
  *
- * Get current application API Key
+ * Get current application API Key (OAuth Client key)
  *
  * Return value: API key or NULL if none set
  */
@@ -620,17 +665,21 @@ flickcurl_get_api_key(flickcurl* fc)
  * @fc: flickcurl object
  * @secret: shared secret
  *
- * Set Shared Secret for flickcurl requests
+ * Set legacy Flickr auth secret
  */
 void
 flickcurl_set_shared_secret(flickcurl* fc, const char *secret)
 {
+  size_t len;
+  
 #if FLICKCURL_DEBUG > 1
-  fprintf(stderr, "Secret: '%s'\n", secret);
+  fprintf(stderr, "Legacy Flickr auth Secret: '%s'\n", secret);
 #endif
   if(fc->secret)
     free(fc->secret);
-  fc->secret = strdup(secret);
+  len = strlen(secret);
+  fc->secret = (char*)malloc(len + 1);
+  memcpy(fc->secret, secret, len + 1);
 }
 
 
@@ -638,7 +687,7 @@ flickcurl_set_shared_secret(flickcurl* fc, const char *secret)
  * flickcurl_get_shared_secret:
  * @fc: flickcurl object
  *
- * Get current Shared Secret
+ * Get legacy Flickr auth Secret
  *
  * Return value: shared secret or NULL if none set
  */
@@ -654,17 +703,22 @@ flickcurl_get_shared_secret(flickcurl* fc)
  * @fc: flickcurl object
  * @auth_token: auth token
  *
- * Set Auth Token for flickcurl requests
+ * Set legacy Flickr auth Token
  */
 void
 flickcurl_set_auth_token(flickcurl *fc, const char* auth_token)
 {
+  size_t len;
+  
 #if FLICKCURL_DEBUG > 1
-  fprintf(stderr, "Auth token: '%s'\n", auth_token);
+  fprintf(stderr, "Legacy Flickr auth token: '%s'\n", auth_token);
 #endif
   if(fc->auth_token)
     free(fc->auth_token);
-  fc->auth_token = strdup(auth_token);
+
+  len = strlen(auth_token);
+  fc->auth_token = (char*)malloc(len + 1);
+  memcpy(fc->auth_token, auth_token, len + 1);
 }
 
 
@@ -672,7 +726,7 @@ flickcurl_set_auth_token(flickcurl *fc, const char* auth_token)
  * flickcurl_get_auth_token:
  * @fc: flickcurl object
  *
- * Get current auth token
+ * Get legacy Flickr auth Token
  *
  * Return value: auth token or NULL if none set
  */
@@ -711,17 +765,35 @@ flickcurl_set_request_delay(flickcurl *fc, long delay_msec)
 }
 
 
-static int
-compare_args(const void *a, const void *b) 
+/*
+ * INTERNAL: initialise parameter array
+ */
+void
+flickcurl_init_params(flickcurl *fc)
 {
-  return strcmp(*(char**)a, *(char**)b);
+  fc->count = 0;
+  fc->parameters[fc->count][0] = NULL;
 }
 
 
-static void
-flickcurl_sort_args(flickcurl *fc, const char *parameters[][2], int count)
+/*
+ * INTERNAL: add a new (key, value) to array of parameters
+ */
+void
+flickcurl_add_param(flickcurl *fc, const char* key, const char* value)
 {
-  qsort((void*)parameters, count, sizeof(char*[2]), compare_args);
+  fc->parameters[fc->count][0] = key;
+  fc->parameters[fc->count][1] = value;
+  fc->count++;
+}
+
+/*
+ * INTERNAL: finish parameters
+ */
+void
+flickcurl_end_params(flickcurl *fc)
+{
+  fc->parameters[fc->count][0] = NULL;
 }
 
 
@@ -729,228 +801,32 @@ static int
 flickcurl_prepare_common(flickcurl *fc, 
                          const char* url,
                          const char* method,
-                         const char* upload_field, const char* upload_value,
-                         const char* parameters[][2], int count,
+                         const char* upload_field,
+                         const char* upload_value,
                          int parameters_in_url, int need_auth)
 {
-  int i;
-  char *md5_string = NULL;
-  size_t* values_len = NULL;
-  unsigned int fc_uri_len = 0;
-  
-  if(!url || !parameters)
-    return 1;
-  
-  /* If one is given, both are required */
-  if((upload_field || upload_value) && (!upload_field || !upload_value))
-    return 1;
-  
-  fc->failed = 0;
-  fc->error_code = 0;
-  if(fc->error_msg) {
-    free(fc->error_msg);
-    fc->error_msg = NULL;
-  }
-  /* Default to read */
-  fc->is_write = 0;
-  /* Default to no data */
-  if(fc->data) {
-    if(fc->data_is_xml)
-      xmlFree(fc->data);
-    fc->data = NULL;
-    fc->data_length = 0;
-    fc->data_is_xml = 0;
-  }
-  if(fc->param_fields) {
-    for(i = 0; fc->param_fields[i]; i++) {
-      free(fc->param_fields[i]);
-      free(fc->param_values[i]);
-    }
-    free(fc->param_fields);
-    free(fc->param_values);
-    fc->param_fields = NULL;
-    fc->param_values = NULL;
-    fc->parameter_count = 0;
-  }
-  if(fc->upload_field) {
-    free(fc->upload_field);
-    fc->upload_field = NULL;
-  }
-  if(fc->upload_value) {
-    free(fc->upload_value);
-    fc->upload_value = NULL;
-  }
-  
-  if(!fc->secret) {
-    flickcurl_error(fc, "No shared secret");
-    return 1;
-  }
-  if(!fc->api_key) {
-    flickcurl_error(fc, "No API key");
-    return 1;
-  }
+  int rc = 1;
 
-  
-  if(fc->method)
-    free(fc->method);
-  if(method)
-    fc->method = strdup(method);
+  if(fc->api_key && fc->secret)
+    /* Call with legacy Flickr auth */
+    rc = flickcurl_legacy_prepare_common(fc, url, method,
+                                         upload_field, upload_value,
+                                         parameters_in_url, need_auth);
+  else if(fc->od.token && fc->od.token_secret)
+    /* Call with OAuth */
+    rc = flickcurl_oauth_prepare_common(fc, url, method,
+                                        upload_field, upload_value,
+                                        parameters_in_url, need_auth);
   else
-    fc->method = NULL;
+    flickcurl_error(fc, "No legacy or OAuth authentication tokens or secrets");
 
-  if(fc->method) {
-    parameters[count][0]  = "method";
-    parameters[count++][1]= fc->method;
-  }
-
-  parameters[count][0]  = "api_key";
-  parameters[count++][1]= fc->api_key;
-
-  if(need_auth && fc->auth_token) {
-    parameters[count][0]  = "auth_token";
-    parameters[count++][1]= fc->auth_token;
-  }
-
-  parameters[count][0]  = NULL;
-
-  /* +1 for api_sig +1 for NULL terminating pointer */
-  fc->param_fields = (char**)calloc(count+2, sizeof(char*));
-  fc->param_values = (char**)calloc(count+2, sizeof(char*));
-  values_len = (size_t*)calloc(count+2, sizeof(size_t));
-
-  if((need_auth && fc->auth_token) || fc->sign)
-    flickcurl_sort_args(fc, parameters, count);
-
-  fc_uri_len = strlen(url);
-  
-  /* Save away the parameters and calculate the value lengths */
-  for(i = 0; parameters[i][0]; i++) {
-    size_t param_len = strlen(parameters[i][0]);
-
-    if(parameters[i][1])
-      values_len[i] = strlen(parameters[i][1]);
-    else {
-      values_len[i] = 0;
-      parameters[i][1] = "";
-    }
-    fc->param_fields[i] = (char*)malloc(param_len+1);
-    strcpy(fc->param_fields[i], parameters[i][0]);
-    fc->param_values[i] = (char*)malloc(values_len[i]+1);
-    strcpy(fc->param_values[i], parameters[i][1]);
-
-    /* 3x value len is conservative URI %XX escaping on every char */
-    fc_uri_len += param_len + 1 /* = */ + 3 * values_len[i];
-  }
-
-  if(upload_field) {
-    fc->upload_field = (char*)malloc(strlen(upload_field)+1);
-    strcpy(fc->upload_field, upload_field);
-
-    fc->upload_value = (char*)malloc(strlen(upload_value)+1);
-    strcpy(fc->upload_value, upload_value);
-  }
-
-  if((need_auth && fc->auth_token) || fc->sign) {
-    size_t buf_len = 0;
-    char *buf;
-    
-    buf_len = strlen(fc->secret);
-    for(i = 0; parameters[i][0]; i++)
-      buf_len += strlen(parameters[i][0]) + values_len[i];
-
-    buf = (char*)malloc(buf_len+1);
-    strcpy(buf, fc->secret);
-    for(i = 0; parameters[i][0]; i++) {
-      strcat(buf, parameters[i][0]);
-      strcat(buf, parameters[i][1]);
-    }
-    
-#ifdef FLICKCURL_DEBUG
-    fprintf(stderr, "MD5 Buffer '%s'\n", buf);
-#endif
-    md5_string = MD5_string(buf);
-    
-    parameters[count][0]  = "api_sig";
-    parameters[count][1]= md5_string;
-
-    /* Add a new parameter pair */
-    values_len[count] = 32; /* MD5 is always 32 */
-    fc->param_fields[count] = (char*)malloc(7+1); /* 7 = strlen(api_sig) */
-    strcpy(fc->param_fields[count], parameters[count][0]);
-    fc->param_values[count] = (char*)malloc(32+1); /* 32 = MD5 */
-    strcpy(fc->param_values[count], parameters[count][1]);
-
-    fc_uri_len += 7 /* "api_sig" */ + 1 /* = */ + 32 /* MD5 value: never escaped */;
-
-    count++;
-    
-#ifdef FLICKCURL_DEBUG
-    fprintf(stderr, "Signature: '%s'\n", parameters[count-1][1]);
-#endif
-    
-    free(buf);
-    
-    parameters[count][0] = NULL;
-  }
-
-  /* add &s between parameters */
-  fc_uri_len += count-1;
-
-  /* reuse or grow uri buffer */
-  if(fc->uri_len < fc_uri_len) {
-    free(fc->uri);
-    fc->uri = (char*)malloc(fc_uri_len+1);
-    fc->uri_len = fc_uri_len;
-  }
-  strcpy(fc->uri, url);
-
-  if(parameters_in_url) {
-    for(i = 0; parameters[i][0]; i++) {
-      char *value = (char*)parameters[i][1];
-      char *escaped_value = NULL;
-
-      if(!parameters[i][1])
-        continue;
-
-      strcat(fc->uri, parameters[i][0]);
-      strcat(fc->uri, "=");
-      if(!strcmp(parameters[i][0], "method")) {
-        /* do not touch method name */
-      } else
-        escaped_value = curl_escape(value, values_len[i]);
-
-      if(escaped_value) {
-        strcat(fc->uri, escaped_value);
-        curl_free(escaped_value);
-      } else
-        strcat(fc->uri, value);
-      strcat(fc->uri, "&");
-    }
-
-    /* zap last & */
-    fc->uri[strlen(fc->uri)-1]= '\0';
-  }
-
-#ifdef FLICKCURL_DEBUG
-  fprintf(stderr, "URI is '%s'\n", fc->uri);
-
-  FLICKCURL_ASSERT((strlen(fc->uri) == fc_uri_len),
-                   "Final URI does not match expected length");
-#endif
-
-  if(md5_string)
-    free(md5_string);
-
-  if(values_len)
-    free(values_len);
-
-  return 0;
+  return rc;
 }
 
 
+
 int
-flickcurl_prepare_noauth(flickcurl *fc, const char* method,
-                         const char* parameters[][2], int count)
+flickcurl_prepare_noauth(flickcurl *fc, const char* method)
 {
   if(!method) {
     flickcurl_error(fc, "No method to prepare");
@@ -961,14 +837,12 @@ flickcurl_prepare_noauth(flickcurl *fc, const char* method,
                                   fc->service_uri,
                                   method,
                                   NULL, NULL,
-                                  parameters, count,
                                   1, 0);
 }
 
 
 int
-flickcurl_prepare(flickcurl *fc, const char* method,
-                  const char* parameters[][2], int count)
+flickcurl_prepare(flickcurl *fc, const char* method)
 {
   if(!method) {
     flickcurl_error(fc, "No method to prepare");
@@ -979,7 +853,6 @@ flickcurl_prepare(flickcurl *fc, const char* method,
                                   fc->service_uri,
                                   method,
                                   NULL, NULL,
-                                  parameters, count,
                                   1, 1);
 }
 
@@ -987,14 +860,12 @@ flickcurl_prepare(flickcurl *fc, const char* method,
 int
 flickcurl_prepare_upload(flickcurl *fc, 
                          const char* url,
-                         const char* upload_field, const char* upload_value,
-                         const char* parameters[][2], int count)
+                         const char* upload_field, const char* upload_value)
 {
   return flickcurl_prepare_common(fc,
                                   url,
                                   NULL,
                                   upload_field, upload_value,
-                                  parameters, count,
                                   0, 1);
 }
 
@@ -1118,11 +989,11 @@ flickcurl_curl_header_callback(void* ptr,  size_t  size, size_t nmemb,
   if(!strncmp((char*)ptr, "X-FlickrErrCode: ", EC_HEADER_LEN)) {
     fc->error_code = atoi((char*)ptr+EC_HEADER_LEN);
   } else if(!strncmp((char*)ptr, "X-FlickrErrMessage: ", EM_HEADER_LEN)) {
-    int len = bytes-EM_HEADER_LEN;
+    int len = bytes - EM_HEADER_LEN;
     if(fc->error_msg)
       free(fc->error_msg);
-    fc->error_msg = (char*)malloc(len+1);
-    strncpy(fc->error_msg, (char*)ptr+EM_HEADER_LEN, len);
+    fc->error_msg = (char*)malloc(len + 1);
+    memcpy(fc->error_msg, (char*)ptr + EM_HEADER_LEN, len + 1);
     fc->error_msg[len] = '\0';
     while(fc->error_msg[len-1] == '\r' || fc->error_msg[len-1] == '\n') {
       fc->error_msg[len-1] = '\0';
@@ -1221,6 +1092,7 @@ flickcurl_invoke_common(flickcurl *fc, char** content_p, size_t* size_p,
   if(1) {
 #ifdef HAVE_RAPTOR
     char* uri_string;
+    size_t len;
 #endif
     
     if(access(filename, R_OK)) {
@@ -1230,7 +1102,8 @@ flickcurl_invoke_common(flickcurl *fc, char** content_p, size_t* size_p,
     }
 #ifdef HAVE_RAPTOR
     uri_string = raptor_uri_filename_to_uri_string(filename);
-    strcpy(fc->uri, uri_string);
+    len = strlen(uri_string);
+    memcpy(fc->uri, uri_string, len + 1);
     raptor_free_memory(uri_string);
 #else
     sprintf(fc->uri, "file:%s", filename);
@@ -1542,8 +1415,11 @@ flickcurl_invoke_common(flickcurl *fc, char** content_p, size_t* size_p,
         const char *attr_value = (const char*)attr->children->content;
         if(!strcmp(attr_name, "code"))
           fc->error_code = atoi(attr_value);
-        else if(!strcmp(attr_name, "msg"))
-          fc->error_msg = strdup(attr_value);
+        else if(!strcmp(attr_name, "msg")) {
+          size_t attr_len = strlen(attr_value);
+          fc->error_msg = (char*)malloc(attr_len + 1);
+          memcpy(fc->error_msg, attr_value, attr_len + 1);
+        }
       }
       if(fc->method)
         flickcurl_error(fc, "Method %s failed with error %d - %s", 
@@ -1684,7 +1560,7 @@ flickcurl_unixtime_to_isotime(time_t unix_time)
   strftime(date_buffer, len+1, ISO_DATE_FORMAT, structured_time);
   
   value = (char*)malloc(len + 1);
-  strncpy((char*)value, date_buffer, len+1);
+  memcpy(value, date_buffer, len + 1);
   return value;
 }
 
@@ -1704,7 +1580,7 @@ flickcurl_unixtime_to_sqltimestamp(time_t unix_time)
   strftime(date_buffer, len+1, SQL_DATETIME_FORMAT, structured_time);
   
   value = (char*)malloc(len + 1);
-  strncpy((char*)value, date_buffer, len+1);
+  memcpy(value, date_buffer, len + 1);
   return value;
 }
 
@@ -1713,20 +1589,21 @@ char*
 flickcurl_sqltimestamp_to_isotime(const char* timestamp)
 {
 /* SQL DATETIME FORMAT "%Y %m %d %H:%M:%S"  (19 chars) */
+#define SQL_DATE_LEN 19
 /* ISO DATE FORMAT     "%Y-%m-%dT%H:%M:%SZ" (20 chars) */
 #define ISO_DATE_LEN 20
   size_t len = ISO_DATE_LEN;
   char *value = NULL;
   
   value = (char*)malloc(len + 1);
-  strncpy((char*)value, timestamp, len);
+  memcpy(value, timestamp, SQL_DATE_LEN);
   value[4] = '-';
   value[7] = '-';
   value[10] = 'T';
   value[13] = ':';
   value[16] = ':';
   value[19] = 'Z';
-  value[20] = '\0';
+  value[ISO_DATE_LEN] = '\0';
   
   return value;
 }
@@ -1762,8 +1639,11 @@ flickcurl_xpath_eval(flickcurl *fc, xmlXPathContextPtr xpathCtx,
       fc->failed = 1;
       break;
     }
-    if(node->children)
-      value = strdup((char*)node->children->content);
+    if(node->children) {
+      size_t len = strlen((char*)node->children->content);
+      value = (char*)malloc(len + 1);
+      memcpy(value, node->children->content, len + 1);
+    }
     break;
   }
 
@@ -1817,7 +1697,7 @@ flickcurl_xpath_eval_to_tree_string(flickcurl* fc,
   value = (char*)malloc(value_len+1);
   if(!value)
     goto tidy;
-  memcpy(value, xmlBufferContent(buffer), value_len+1);
+  memcpy(value, xmlBufferContent(buffer), value_len + 1);
 
   tidy:
   if(buffer)
@@ -1937,20 +1817,18 @@ flickcurl_call_get_one_string_field(flickcurl* fc,
                                     const char* method,
                                     const xmlChar* xpathExpr)
 {
-  const char * parameters[6][2];
-  int count = 0;
   char *result = NULL;
   xmlDocPtr doc = NULL;
   xmlXPathContextPtr xpathCtx = NULL; 
 
+  flickcurl_init_params(fc);  
   if(key && value) {
-    parameters[count][0]  = key;
-    parameters[count++][1]= value;
+    flickcurl_add_param(fc, key, value);
   }
 
-  parameters[count][0]  = NULL;
+  flickcurl_end_params(fc);
 
-  if(flickcurl_prepare(fc, method, parameters, count))
+  if(flickcurl_prepare(fc, method))
     goto tidy;
 
   doc = flickcurl_invoke(fc);
@@ -1998,8 +1876,8 @@ flickcurl_array_join(const char *array[], char delim)
   p = str;
   for(i = 0; array[i]; i++) {
     size_t item_len = strlen(array[i]);
-    strncpy(p, array[i], item_len);
-    p+= item_len;
+    memcpy(p, array[i], item_len);
+    p += item_len;
     if(i < array_size)
       *p++ = delim;
   }
@@ -2040,14 +1918,15 @@ flickcurl_array_split(const char *str, char delim)
 
     for(p = str; *p && *p != delim; p++)
       ;
-    item_len = p-str;
-    array[i] = (char*)malloc(item_len+1);
+    item_len = p - str;
+    array[i] = (char*)malloc(item_len + 1);
     if(!array[i]) {
       while(--i >= 0)
         free(array[i]);
+      free(array);
       return NULL;
     }
-    strncpy(array[i], str, item_len);
+    memcpy(array[i], str, item_len);
     array[i][item_len] = '\0';
     str+= item_len;
     if(*str == delim)
@@ -2304,9 +2183,8 @@ flickcurl_get_feed_format_info(int feed_format,
 
 /*
  * flickcurl_append_photos_list_params:
+ * @fc: fc
  * @list_params: in parameter - photos list paramater
- * @parameters: in/out parameter - array of name/value parameters
- * @count_p: in/out parameter - updated as new parameters added
  * @format_p: out parameter - result format requested or NULL
  *
  * INTERNAL - append #flickcurl_photos_list_params to parameter list for API call
@@ -2314,14 +2192,14 @@ flickcurl_get_feed_format_info(int feed_format,
  * Return value: number of parameters added
  */
 int
-flickcurl_append_photos_list_params(flickcurl_photos_list_params* list_params,
-                                    const char* parameters[][2], int* count_p,
+flickcurl_append_photos_list_params(flickcurl* fc,
+                                    flickcurl_photos_list_params* list_params,
                                     const char** format_p)
 {
   /* NOTE: These are SHARED and pointed to by flickcurl_prepare() to
    * build the URL */
-  static char per_page_s[4];
-  static char page_s[4];
+  static char per_page_s[FLICKCURL_MAX_LIST_PARAM_COUNT];
+  static char page_s[FLICKCURL_MAX_LIST_PARAM_COUNT];
   int this_count = 0;
   
   if(format_p)
@@ -2331,33 +2209,25 @@ flickcurl_append_photos_list_params(flickcurl_photos_list_params* list_params,
     return 0;
   
   if(list_params->extras) {
-    parameters[*count_p][0]  = "extras";
-    parameters[*count_p][1]= list_params->extras;
-    (*count_p)++;
+    flickcurl_add_param(fc, "extras", list_params->extras);
     this_count++;
   }
   if(list_params->per_page) {
     if(list_params->per_page >= 0 && list_params->per_page <= 999) {
       sprintf(per_page_s, "%d", list_params->per_page);
-      parameters[*count_p][0]  = "per_page";
-      parameters[*count_p][1]= per_page_s;
-      (*count_p)++;
+      flickcurl_add_param(fc, "per_page", per_page_s);
       this_count++;
     }
   }
   if(list_params->page) {
     if(list_params->page >= 0 && list_params->page <= 999) {
       sprintf(page_s, "%d", list_params->page);
-      parameters[*count_p][0]  = "page";
-      parameters[*count_p][1]= page_s;
-      (*count_p)++;
+      flickcurl_add_param(fc, "page", page_s);
       this_count++;
     }
   }
   if(list_params->format) {
-    parameters[*count_p][0]  = "format";
-    parameters[*count_p][1]= list_params->format;
-    (*count_p)++;
+    flickcurl_add_param(fc, "format", list_params->format);
     this_count++;
 
     if(format_p)
