@@ -5,7 +5,7 @@
  * Commons API announced 2009-01-29
  * http://flickr.com/groups/api/discuss/72157613093793775/
  *
- * Copyright (C) 2009, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2009-2012, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -58,15 +58,15 @@
 flickcurl_institution**
 flickcurl_commons_getInstitutions(flickcurl* fc)
 {
-  const char* parameters[7][2];
-  int count = 0;
   xmlDocPtr doc = NULL;
   xmlXPathContextPtr xpathCtx = NULL; 
   flickcurl_institution** institutions = NULL;
   
-  parameters[count][0]  = NULL;
+  flickcurl_init_params(fc);
 
-  if(flickcurl_prepare(fc, "flickr.commons.getInstitutions", parameters, count))
+  flickcurl_end_params(fc);
+
+  if(flickcurl_prepare(fc, "flickr.commons.getInstitutions"))
     goto tidy;
 
   doc = flickcurl_invoke(fc);
