@@ -2,7 +2,7 @@
  *
  * oauth.c - OAuth 1.0 for Flickr
  *
- * Copyright (C) 2011, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2011-2012, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -258,9 +258,9 @@ compare_args(const void *a, const void *b)
 
 
 static void
-flickcurl_sort_args(flickcurl *fc, const char *parameters[][2], int count)
+flickcurl_sort_args(flickcurl *fc)
 {
-  qsort((void*)parameters, count, sizeof(char*[2]), compare_args);
+  qsort((void*)fc->parameters, fc->count, sizeof(char*[2]), compare_args);
 }
 
 
@@ -276,7 +276,6 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
                                const char* method,
                                const char* upload_field,
                                const char* upload_value,
-                               const char* parameters[][2], int count,
                                int parameters_in_url, int need_auth)
 {
   flickcurl_oauth_data* od = &fc->od;
@@ -284,6 +283,7 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
   char *signature_string = NULL;
   size_t* values_len = NULL;
   unsigned int fc_uri_len = 0;
+  unsigned int full_uri_len = 0;
   char* nonce = NULL;
   int free_nonce = 0;
   char* timestamp = NULL;
@@ -291,8 +291,9 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
   int need_to_add_query = 0;
   const char* http_method = "GET";
   int is_oauth_method = 0;
+  char *p;
 
-  if(!url || !parameters)
+  if(!url)
     return 1;
   
   /* If one is given, both are required */
@@ -337,8 +338,10 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
   
   if(fc->method)
     free(fc->method);
-  if(method) {
-    fc->method = strdup(method);
+  if(method) { 
+    size_t len = strlen(method);
+    fc->method = (char*)malloc(len + 1);
+    memcpy(fc->method, method, len + 1);
     is_oauth_method = !strncmp(method, "flickr.oauth.", 13);
   } else
     fc->method = NULL;
@@ -357,18 +360,13 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
    * oauth_token            access token or request token
    */
 
-  if(fc->method && !is_oauth_method) {
-    parameters[count][0]  = "method";
-    parameters[count++][1]= fc->method;
-  }
+  if(fc->method && !is_oauth_method)
+    flickcurl_add_param(fc, "method", fc->method);
 
-  if(od->callback) {
-    parameters[count][0]  = "oauth_callback";
-    parameters[count++][1]= od->callback;
-  }
+  if(od->callback)
+    flickcurl_add_param(fc, "oauth_callback", od->callback);
   
-  parameters[count][0]  = "oauth_consumer_key";
-  parameters[count++][1]= od->client_key;
+  flickcurl_add_param(fc, "oauth_consumer_key", od->client_key);
 
   nonce = (char*)od->nonce;
   if(!nonce) {
@@ -376,12 +374,10 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
     free_nonce = 1;
     sprintf(nonce, "%ld", mtwist_u32rand(fc->mt));
   }
-  parameters[count][0]  = "oauth_nonce";
-  parameters[count++][1]= nonce;
+  flickcurl_add_param(fc, "oauth_nonce", nonce);
 
   /* oauth_signature - computed over these fields */
-  parameters[count][0]  = "oauth_signature_method";
-  parameters[count++][1]= "HMAC-SHA1";
+  flickcurl_add_param(fc, "oauth_signature_method", "HMAC-SHA1");
 
   timestamp = (char*)malloc(20);
   if(od->timestamp)
@@ -391,64 +387,63 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
     (void)gettimeofday(&tp, NULL);
     sprintf(timestamp, "%ld", (long)tp.tv_sec);
   }
-  parameters[count][0]  = "oauth_timestamp";
-  parameters[count++][1]= timestamp;
+  flickcurl_add_param(fc, "oauth_timestamp", timestamp);
 
-  parameters[count][0]  = "oauth_version";
-  parameters[count++][1]= "1.0";
+  flickcurl_add_param(fc, "oauth_version", "1.0");
 
-  if(od->token) {
-    parameters[count][0]  = "oauth_token";
-    parameters[count++][1]= od->token;
-  } else if(od->request_token) {
-    parameters[count][0]  = "oauth_token";
-    parameters[count++][1]= od->request_token;
-  }
-  if(od->verifier) {
-    parameters[count][0]  = "oauth_verifier";
-    parameters[count++][1]= od->verifier;
-  }
+  if(od->token)
+    flickcurl_add_param(fc, "oauth_token", od->token);
+  else if(od->request_token) 
+    flickcurl_add_param(fc, "oauth_token", od->request_token);
 
-  parameters[count][0]  = NULL;
+  if(od->verifier)
+    flickcurl_add_param(fc, "oauth_verifier", od->verifier);
+
+  flickcurl_end_params(fc);
 
   /* +FLICKCURL_FLICKCURL_MAX_OAUTH_PARAM_COUNT for oauth fields +1 for NULL terminating pointer */
-  fc->param_fields = (char**)calloc(count + FLICKCURL_MAX_OAUTH_PARAM_COUNT + 1, sizeof(char*));
-  fc->param_values = (char**)calloc(count + FLICKCURL_MAX_OAUTH_PARAM_COUNT + 1, sizeof(char*));
-  values_len       = (size_t*)calloc(count + FLICKCURL_MAX_OAUTH_PARAM_COUNT + 1, sizeof(size_t));
+  fc->param_fields = (char**)calloc(fc->count + FLICKCURL_MAX_OAUTH_PARAM_COUNT + 1, sizeof(char*));
+  fc->param_values = (char**)calloc(fc->count + FLICKCURL_MAX_OAUTH_PARAM_COUNT + 1, sizeof(char*));
+  values_len       = (size_t*)calloc(fc->count + FLICKCURL_MAX_OAUTH_PARAM_COUNT + 1, sizeof(size_t));
 
   if((need_auth && (od->client_secret || od->token_secret)) || fc->sign)
-    flickcurl_sort_args(fc, parameters, count);
+    flickcurl_sort_args(fc);
 
 
   fc_uri_len = strlen(url);
-  if(url[fc_uri_len -1] != '?')
+  full_uri_len = fc_uri_len;
+
+  if(url[fc_uri_len - 1] != '?')
     need_to_add_query++;
   
   /* Save away the parameters and calculate the value lengths */
-  for(i = 0; parameters[i][0]; i++) {
-    size_t param_len = strlen(parameters[i][0]);
+  for(i = 0; fc->parameters[i][0]; i++) {
+    size_t param_len = strlen(fc->parameters[i][0]);
 
-    if(parameters[i][1])
-      values_len[i] = strlen(parameters[i][1]);
+    if(fc->parameters[i][1])
+      values_len[i] = strlen(fc->parameters[i][1]);
     else {
       values_len[i] = 0;
-      parameters[i][1] = "";
+      fc->parameters[i][1] = "";
     }
     fc->param_fields[i] = (char*)malloc(param_len + 1);
-    strcpy(fc->param_fields[i], parameters[i][0]);
+    memcpy(fc->param_fields[i], fc->parameters[i][0], param_len + 1);
+
     fc->param_values[i] = (char*)malloc(values_len[i] + 1);
-    strcpy(fc->param_values[i], parameters[i][1]);
+    memcpy(fc->param_values[i], fc->parameters[i][1], values_len[i] + 1);
 
     /* 3x value len is conservative URI %XX escaping on every char */
-    fc_uri_len += param_len + 1 /* = */ + 3 * values_len[i];
+    full_uri_len += param_len + 1 /* = */ + 3 * values_len[i];
   }
 
   if(upload_field) {
-    fc->upload_field = (char*)malloc(strlen(upload_field) + 1);
-    strcpy(fc->upload_field, upload_field);
+    size_t len = strlen(upload_field);
+    fc->upload_field = (char*)malloc(len + 1);
+    memcpy(fc->upload_field, upload_field, len + 1);
 
-    fc->upload_value = (char*)malloc(strlen(upload_value) + 1);
-    strcpy(fc->upload_value, upload_value);
+    len = strlen(upload_value);
+    fc->upload_value = (char*)malloc(len + 1);
+    memcpy(fc->upload_value, upload_value, len + 1);
   }
 
 
@@ -460,39 +455,64 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
     size_t param_buf_len = 0;
     size_t vlen = 0;
     char *escaped_value = NULL;
+    size_t http_method_len;
+    size_t escaped_value_len;
     
-    for(i = 0; parameters[i][0]; i++)
-      param_buf_len += strlen(parameters[i][0]) + 3 + (3 * values_len[i]) + 3;
+    for(i = 0; fc->parameters[i][0]; i++)
+      param_buf_len += strlen(fc->parameters[i][0]) + 3 + (3 * values_len[i]) + 3;
     param_buf = (char*)malloc(param_buf_len + 1);
     *param_buf = '\0';
     
-    for(i = 0; parameters[i][0]; i++) {
+    p = param_buf;
+
+    for(i = 0; fc->parameters[i][0]; i++) {
+      size_t len = strlen(fc->parameters[i][0]);
       if(i > 0)
-        strcat(param_buf, "&");
-      strcat(param_buf, parameters[i][0]);
-      strcat(param_buf, "=");
-      escaped_value = curl_escape(parameters[i][1], 0);
-      strcat(param_buf, escaped_value);
+        *p++ = '&';
+      memcpy(p, fc->parameters[i][0], len);
+      p += len;
+
+      *p++ = '=';
+
+      escaped_value = curl_escape(fc->parameters[i][1], 0);
+      escaped_value_len = strlen(escaped_value);
+      memcpy(p, escaped_value, escaped_value_len);
+      p += escaped_value_len;
       curl_free(escaped_value);
     }
+    *p = '\0';
 
-    buf_len = strlen(http_method);
+    http_method_len = strlen(http_method);
+
+    buf_len = http_method_len;
     buf_len += 1; /* & */
     buf_len += (3 * strlen(url));
     buf_len += 1; /* & */
     buf_len += param_buf_len * 3;
 
     buf = (char*)malloc(buf_len + 1);
-    strcpy(buf, http_method);
-    strcat(buf, "&");
+
+    p = buf;
+    memcpy(p, http_method, http_method_len);
+    p += http_method_len;
+
+    *p++ = '&';
+
     escaped_value = curl_escape(url, 0);
-    strcat(buf, escaped_value);
-    curl_free(escaped_value);
-    strcat(buf, "&");
-    escaped_value = curl_escape(param_buf, 0);
-    strcat(buf, escaped_value);
+    escaped_value_len = strlen(escaped_value);
+    memcpy(p, escaped_value, escaped_value_len);
+    p += escaped_value_len;
     curl_free(escaped_value);
 
+    *p++ = '&';
+
+    escaped_value = curl_escape(param_buf, 0);
+    escaped_value_len = strlen(escaped_value);
+    memcpy(p, escaped_value, escaped_value_len);
+    p += escaped_value_len;
+    curl_free(escaped_value);
+    *p = '\0';
+    
     free(param_buf);
 
     if(flickcurl_oauth_build_key(od)) {
@@ -517,20 +537,21 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
     free(od->key);
     od->key = NULL;
 
-    parameters[count][0]  = "oauth_signature";
-    parameters[count][1]  = signature_string;
-
+    flickcurl_add_param(fc, "oauth_signature", signature_string);
+    fc->count--;
+    
     /* Add a new parameter pair */
-    values_len[count] = vlen;
+    values_len[fc->count] = vlen;
     /* 15 = strlen(oauth_signature) */
-    fc->param_fields[count] = (char*)malloc(15 + 1);
-    strcpy(fc->param_fields[count], parameters[count][0]);
-    fc->param_values[count] = (char*)malloc(vlen + 1);
-    strcpy(fc->param_values[count], parameters[count][1]);
+    fc->param_fields[fc->count] = (char*)malloc(15 + 1);
+    memcpy(fc->param_fields[fc->count], fc->parameters[fc->count][0], 15 + 1);
 
-    fc_uri_len += 15 /* "oauth_signature" */ + 1 /* = */ + vlen;
+    fc->param_values[fc->count] = (char*)malloc(vlen + 1);
+    memcpy(fc->param_values[fc->count], fc->parameters[fc->count][1], vlen + 1);
 
-    count++;
+    full_uri_len += 15 /* "oauth_signature" */ + 1 /* = */ + vlen;
+    
+    fc->count++;
     
 #ifdef FLICKCURL_DEBUG
     fprintf(stderr, "HMAC-SHA1 signature:\n  %s\n", signature_string);
@@ -540,41 +561,50 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
     od->data = NULL;
     od->data_len = 0;
     
-    parameters[count][0] = NULL;
+    flickcurl_end_params(fc);
   }
 
-  /* add &s between parameters */
-  fc_uri_len += count-1;
+  /* add &s between fc->parameters */
+  full_uri_len += fc->count - 1;
 
   /* reuse or grow uri buffer */
-  if(fc->uri_len < fc_uri_len) {
+  if(fc->uri_len < full_uri_len) {
     free(fc->uri);
-    fc->uri = (char*)malloc(fc_uri_len+1);
-    fc->uri_len = fc_uri_len;
+    fc->uri = (char*)malloc(full_uri_len + 1);
+    fc->uri_len = full_uri_len;
   }
-  strcpy(fc->uri, url);
+  memcpy(fc->uri, url, fc_uri_len);
+
+  p = fc->uri + fc_uri_len;
+  *p = '\0';
 
   if(need_to_add_query)
-    strcat(fc->uri, "?");
+    *p++ = '?';
 
   if(parameters_in_url) {
-    for(i = 0; parameters[i][0]; i++) {
-      char *value = (char*)parameters[i][1];
-      char *escaped_value = NULL;
+    for(i = 0; fc->parameters[i][0]; i++) {
+      char *value = (char*)fc->parameters[i][1];
+      size_t len;
 
-      if(!parameters[i][1])
+      if(!fc->parameters[i][1])
         continue;
 
-      strcat(fc->uri, parameters[i][0]);
-      strcat(fc->uri, "=");
-      escaped_value = curl_escape(value, values_len[i]);
-      strcat(fc->uri, escaped_value);
-      curl_free(escaped_value);
-      strcat(fc->uri, "&");
+      len = strlen(fc->parameters[i][0]);
+      memcpy(p, fc->parameters[i][0], len);
+      p += len;
+      *p++ = '=';
+
+      value = curl_escape(value, values_len[i]);
+      len = strlen(value);
+      memcpy(p, value, len);
+      p += len;
+      curl_free(value);
+
+      *p++ = '&';
     }
 
-    /* zap last & */
-    fc->uri[strlen(fc->uri)-1] = '\0';
+    /* zap last & and terminate fc->url */
+    *--p = '\0';
   }
 
 #ifdef FLICKCURL_DEBUG
@@ -602,44 +632,50 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
 
 
 
-/*
- * flickcurl_oauth_request_token:
+/**
+ * flickcurl_oauth_create_request_token:
  * @fc: flickcurl object
+ * @callback: callback URL or NULL for out of band
  *
- * INTERNAL - get a Flickr OAuth request token
+ * Request an OAuth request token from Flickr for the application API Key/secret
  *
- * Calls the Flickr OAuth endpoint to get a request token.
+ * Requires the OAuth Client key (API key) and Client secret to have
+ * been set with flickcurl_set_oauth_client_key() and
+ * flickcurl_set_oauth_client_secret() respectively.
+ * 
+ * Calls the Flickr OAuth endpoint to get a request token for the
+ * given callback or uses out-of-band if @callback is NULL.
  *
- * Stores the request token in @od fields 'request_token' and
- * 'request_token_secret' on success.
+ * On success, stores the request token in the @fc structure.
  *
  * Return value: non-0 on failure
  */
 int
-flickcurl_oauth_request_token(flickcurl* fc)
+flickcurl_oauth_create_request_token(flickcurl* fc, const char* callback)
 {
   flickcurl_oauth_data* od = &fc->od;
-  const char * parameters[2 + FLICKCURL_MAX_OAUTH_PARAM_COUNT][2];
-  int count = 0;
   char* request_token = NULL;
   char* request_token_secret = NULL;
   char** form = NULL;
   int rc = 0;
   const char* uri = fc->oauth_request_token_uri;
   int i;
+  int count;
 
-  parameters[count][0]  = NULL;
+  flickcurl_end_params(fc);
 
   /* Require signature */
   flickcurl_set_sign(fc);
 
-  od->callback = "oob";
+  if(!callback || !*callback)
+    callback = "oob";
+  od->callback = callback;
+
   rc = flickcurl_oauth_prepare_common(fc,
                                       uri,
                                       /* method */ "flickr.oauth.request_token",
                                       /* upload_field */ NULL,
                                       /* upload_value */ NULL,
-                                      parameters, count,
                                       /* parameters_in_url */ 1,
                                       /* need_auth */ 1);
   od->callback = NULL;
@@ -668,10 +704,15 @@ flickcurl_oauth_request_token(flickcurl* fc)
 
   if(request_token && request_token_secret) {
     /* Take copies that are owned by od */
-    od->request_token = strdup(request_token);
-    od->request_token_len = strlen(od->request_token);
-    od->request_token_secret = strdup(request_token_secret);
-    od->request_token_secret_len = strlen(od->request_token_secret);
+    size_t len = strlen(request_token);
+    od->request_token = (char*)malloc(len + 1);
+    memcpy(od->request_token, request_token, len + 1);
+    od->request_token_len = len;
+
+    len = strlen(request_token_secret);
+    od->request_token_secret = (char*)malloc(len + 1);
+    memcpy(od->request_token_secret, request_token_secret, len + 1);
+    od->request_token_secret_len = len;
 
 #ifdef FLICKCURL_DEBUG
     fprintf(stderr,
@@ -689,15 +730,21 @@ flickcurl_oauth_request_token(flickcurl* fc)
 }
 
 
-/*
+/**
  * flickcurl_oauth_get_authorize_uri:
  * @fc: flickcurl object
  *
- * INTERNAL - get the URL for the user to authorize an application
+ * Get the URL for the user to authorize Flickr OAuth of an application
  *
  * Forms the URL the user needs to start at to authorize the
- * application.  The application should pass the verifier to
- * flickcurl_oauth_access_token() for the final step in OAuth.
+ * application.
+ *
+ * This function requires flickcurl_oauth_create_request_token() to have
+ * been called to build a request token / secret pair.
+ *
+ * After this function, the application should pass the verifier to
+ * flickcurl_oauth_create_access_token() for the final step in OAuth along
+ * with the request token and request token secret.
  *
  * Return value: authorize URI or NULL on failure
  */
@@ -732,38 +779,35 @@ flickcurl_oauth_get_authorize_uri(flickcurl* fc)
 }
 
 
-/*
- * flickcurl_oauth_access_token:
+/**
+ * flickcurl_oauth_create_access_token:
  * @fc: flickcurl object
  * @verifier: verifier from OOB authentication
  *
- * INTERNAL - get a Flickr OAuth access token from a verifier
+ * Get a Flickr OAuth access token from request token credentials and verifier
  *
  * Calls the Flickr OAuth access token endpoint using the verifier
- * from out of band authentication to get an access token.
- *
- * Uses the @verifier and the @od fields 'request_token' and
- * 'request_token_secret' to get an access token stored which on
- * success is stored in the @od fields 'access_token' and
- * 'access_token_secret'.  The request token fields are deleted on
- * success.
+ * from out of band authentication to get an access token to enable
+ * authenticated calls to the Flickr API.
  *
  * Return value: non-0 on failure
  */
 int
-flickcurl_oauth_access_token(flickcurl* fc, const char* verifier)
+flickcurl_oauth_create_access_token(flickcurl* fc, const char* verifier)
 {
   flickcurl_oauth_data* od = &fc->od;
-  const char * parameters[2 + FLICKCURL_MAX_OAUTH_PARAM_COUNT][2];
-  int count = 0;
   char* access_token = NULL;
   char* access_token_secret = NULL;
   char** form = NULL;
   int rc = 0;
   const char* uri = fc->oauth_access_token_uri;
   int i;
+  int count;
   
-  parameters[count][0]  = NULL;
+  if(!verifier)
+    return 1;
+
+  flickcurl_end_params(fc);
 
   /* Require signature */
   flickcurl_set_sign(fc);
@@ -776,7 +820,6 @@ flickcurl_oauth_access_token(flickcurl* fc, const char* verifier)
                                       /* method */ "flickr.oauth.access_token",
                                       /* upload_field */ NULL,
                                       /* upload_value */ NULL,
-                                      parameters, count,
                                       /* parameters_in_url */ 1,
                                       /* need_auth */ 1);
 
@@ -807,10 +850,15 @@ flickcurl_oauth_access_token(flickcurl* fc, const char* verifier)
 
   if(access_token && access_token_secret) {
     /* Take copies that are owned by od */
-    od->token = strdup(access_token);
-    od->token_len = strlen(od->token);
-    od->token_secret = strdup(access_token_secret);
-    od->token_secret_len = strlen(od->token_secret);
+    size_t len = strlen(access_token);
+    od->token = (char*)malloc(len + 1);
+    memcpy(od->token, access_token, len + 1);
+    od->token_len = len;
+
+    len = strlen(access_token_secret);
+    od->token_secret = (char*)malloc(len + 1);
+    memcpy(od->token_secret, access_token_secret, len + 1);
+    od->token_secret_len = len;
 
     /* Delete temporary request token and secret */
     free(od->request_token);
@@ -835,6 +883,273 @@ flickcurl_oauth_access_token(flickcurl* fc, const char* verifier)
   
   return rc;
 }
+
+
+
+/**
+ * flickcurl_get_oauth_client_key:
+ * @fc: flickcurl object
+ *
+ * Get OAuth Client Key (aka Consumer key, API Key)
+ *
+ * See also flickcurl_get_oauth_client_secret()
+ *
+ * Return value: client key or NULL if none set
+ */
+const char*
+flickcurl_get_oauth_client_key(flickcurl *fc)
+{
+  return fc->od.client_key;
+}
+
+
+/**
+ * flickcurl_get_oauth_client_secret:
+ * @fc: flickcurl object
+ *
+ * Get OAuth Client secret
+ *
+ * See also flickcurl_get_oauth_client_key()
+ *
+ * Return value: client secret or NULL if none set
+ */
+const char*
+flickcurl_get_oauth_client_secret(flickcurl *fc)
+{
+  return fc->od.client_secret;
+}
+
+
+/**
+ * flickcurl_set_oauth_client_key:
+ * @fc: flickcurl object
+ * @client_key: client key (API key)
+ *
+ * Set OAuth client key (aka API key)
+ *
+ * See also flickcurl_get_oauth_client_key()
+ */
+void
+flickcurl_set_oauth_client_key(flickcurl *fc, const char* client_key)
+{
+  if(fc->od.client_key) {
+    free(fc->od.client_key);
+    fc->od.client_key = NULL;
+    fc->od.client_key_len = 0;
+  }
+
+  if(client_key) {
+    size_t len = strlen(client_key);
+    fc->od.client_key = (char*)malloc(len + 1);
+    memcpy(fc->od.client_key, client_key, len + 1);
+    fc->od.client_key_len = len;
+  }
+}
+
+
+/**
+ * flickcurl_set_oauth_client_secret:
+ * @fc: flickcurl object
+ * @client_secret: client key (shared key)
+ *
+ * Set OAuth client key (aka shared secret)
+ *
+ * See also flickcurl_set_oauth_client_secret()
+ */
+void
+flickcurl_set_oauth_client_secret(flickcurl *fc, const char* client_secret)
+{
+  if(fc->od.client_secret) {
+    free(fc->od.client_secret);
+    fc->od.client_secret = NULL;
+    fc->od.client_secret_len = 0;
+  }
+  
+  if(client_secret) {
+    size_t len = strlen(client_secret);
+    fc->od.client_secret = (char*)malloc(len + 1);
+    memcpy(fc->od.client_secret, client_secret, len + 1);
+    fc->od.client_secret_len = len;
+  }
+}
+
+
+/**
+ * flickcurl_get_oauth_token:
+ * @fc: flickcurl object
+ *
+ * Get OAuth Token
+ *
+ * Return value: auth token or NULL if none set
+ */
+const char*
+flickcurl_get_oauth_token(flickcurl *fc)
+{
+  return fc->od.token;
+}
+
+
+/**
+ * flickcurl_set_oauth_token:
+ * @fc: flickcurl object
+ * @token: auth token
+ *
+ * Set OAuth Token
+ */
+void
+flickcurl_set_oauth_token(flickcurl *fc, const char* token)
+{
+#if FLICKCURL_DEBUG > 1
+  fprintf(stderr, "OAuth token: '%s'\n", token);
+#endif
+  if(fc->od.token) {
+    free(fc->od.token);
+    fc->od.token = NULL;
+    fc->od.token_len = 0;
+  }
+  if(token) {
+    size_t len = strlen(token);
+    fc->od.token = (char*)malloc(len + 1);
+    memcpy(fc->od.token, token, len + 1);
+    fc->od.token_len = len;
+  }
+}
+
+
+/**
+ * flickcurl_get_oauth_token_secret:
+ * @fc: flickcurl object
+ *
+ * Get OAuth token secret
+ *
+ * Return value: secret or NULL if none set
+ */
+const char*
+flickcurl_get_oauth_token_secret(flickcurl* fc)
+{
+  return fc->od.token_secret;
+}
+
+
+/**
+ * flickcurl_set_oauth_token_secret:
+ * @fc: flickcurl object
+ * @secret: shared secret
+ *
+ * Set OAuth token Secret
+ */
+void
+flickcurl_set_oauth_token_secret(flickcurl* fc, const char *secret)
+{
+#if FLICKCURL_DEBUG > 1
+  fprintf(stderr, "OAuth token secret: '%s'\n", secret);
+#endif
+  if(fc->od.token_secret) {
+    free(fc->od.token_secret);
+    fc->od.token_secret = NULL;
+    fc->od.token_secret_len = 0;
+  }
+  
+  if(secret) {
+    size_t len = strlen(secret);
+    fc->od.token_secret = (char*)malloc(len + 1);
+    memcpy(fc->od.token_secret, secret, len + 1);
+    fc->od.token_secret_len = len;
+  }
+}
+
+
+/**
+ * flickcurl_get_oauth_request_token:
+ * @fc: flickcurl object
+ *
+ * Get OAuth request token
+ *
+ * Return value: request token or NULL if none set
+ */
+const char*
+flickcurl_get_oauth_request_token(flickcurl* fc)
+{
+  return fc->od.request_token;
+}
+
+
+/**
+ * flickcurl_get_oauth_request_token_secret:
+ * @fc: flickcurl object
+ *
+ * Get OAuth request token secret
+ *
+ * Return value: request token secret or NULL if none set
+ */
+const char*
+flickcurl_get_oauth_request_token_secret(flickcurl* fc)
+{
+  return fc->od.request_token_secret;
+}
+
+
+/**
+ * flickcurl_set_oauth_request_token:
+ * @fc: flickcurl object
+ * @token: request token
+ *
+ * Set OAuth request token
+ *
+ * See also flickcurl_get_oauth_request_token()
+ */
+void
+flickcurl_set_oauth_request_token(flickcurl *fc, const char* token)
+{
+#if FLICKCURL_DEBUG > 1
+  fprintf(stderr, "OAuth request token: '%s'\n", token);
+#endif
+  if(fc->od.request_token) {
+    free(fc->od.request_token);
+    fc->od.request_token = NULL;
+    fc->od.request_token_len = 0;
+  }
+
+  if(token) {
+    size_t len = strlen(token);
+    fc->od.request_token = (char*)malloc(len + 1);
+    memcpy(fc->od.request_token, token, len + 1);
+    fc->od.request_token_len = len;
+  }
+}
+
+
+/**
+ * flickcurl_set_oauth_request_token_secret:
+ * @fc: flickcurl object
+ * @secret: request token secret
+ *
+ * Set OAuth request token secret credentials
+ *
+ * See also flickcurl_get_oauth_request_token_secret()
+ */
+void
+flickcurl_set_oauth_request_token_secret(flickcurl *fc, const char* secret)
+{
+#if FLICKCURL_DEBUG > 1
+  fprintf(stderr, "OAuth request token secret: '%s'\n", secret);
+#endif
+  if(fc->od.request_token_secret) {
+    free(fc->od.request_token_secret);
+    fc->od.request_token_secret = NULL;
+    fc->od.request_token_secret = 0;
+  }
+  if(secret) {
+    size_t len = strlen(secret);
+    fc->od.request_token_secret = (char*)malloc(len + 1);
+    memcpy(fc->od.request_token_secret, secret, len + 1);
+    fc->od.request_token_secret_len = len;
+  }
+}
+
+
+
+
 #endif
 
 
@@ -902,7 +1217,7 @@ test_request_token(flickcurl* fc)
 
   oauth_init_test_secrets(od);
 
-  rc = flickcurl_oauth_request_token(fc);
+  rc = flickcurl_oauth_create_request_token(fc, NULL);
 
   memset(od, '\0', sizeof(*od));
 
@@ -927,7 +1242,7 @@ test_access_token(flickcurl* fc)
   
   oauth_init_test_secrets(od);
 
-  rc = flickcurl_oauth_access_token(fc, verifier);
+  rc = flickcurl_oauth_create_access_token(fc, verifier);
 
   memset(od, '\0', sizeof(*od));
 
@@ -1006,11 +1321,21 @@ test_signature_calc(flickcurl* fc)
                                  test_uri_base_string, 
                                  test_request_parameters);
   
-  fprintf(stderr, "%s: key is (%d bytes)\n  %s\n", program, (int)od->key_len, od->key);
-  fprintf(stderr, "%s: expected key is\n  %s\n", program, expected_key);
-  
-  fprintf(stderr, "%s: data is (%d bytes)\n  %s\n", program, (int)od->data_len, od->data);
-  fprintf(stderr, "%s: expected data is\n  %s\n", program, expected_data);
+  if(strcmp((const char*)od->key, expected_key)) {
+    fprintf(stderr, "%s: FAIL\n" 
+                    "  key is (%d bytes)\n    %s\n"
+                    "  expected key is\n    %s\n",
+            program, (int)od->key_len, od->key, expected_key);
+    rc++;
+  }
+
+  if(strcmp((const char*)od->data, expected_data)) {
+    fprintf(stderr, "%s: FAIL\n"
+            "  data is (%d bytes)\n    %s\n"
+            "  expected data is\n    %s\n",
+            program, (int)od->data_len, od->data, expected_data);
+    rc++;
+  }
   
   signature = flickcurl_oauth_compute_signature(od, &escaped_s_len);
   
@@ -1018,11 +1343,13 @@ test_signature_calc(flickcurl* fc)
   free(signature);
   escaped_s_len = strlen(escaped_s);
   
-  fprintf(stdout, "URI Escaped result (%d bytes):\n   %s\n",
-          (int)escaped_s_len, escaped_s);
-  
-  fprintf(stdout, "Expected URI escaped result\n   %s\n", 
-          expected_signature);
+  if(strcmp(escaped_s, expected_signature)) {
+    fprintf(stdout, "%s: FAIL\n"
+            "  URI Escaped result (%d bytes):\n    %s\n"
+            "  Expected URI escaped result\n    %s\n", 
+            program, (int)escaped_s_len, escaped_s, expected_signature);
+    rc++;
+  }
   
   curl_free(escaped_s);
   
