@@ -2,7 +2,7 @@
  *
  * photos-licenses.c - Flickr flickr.photos.licenses.* API calls
  *
- * Copyright (C) 2007-2008, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2007-2012, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -60,8 +60,6 @@ compare_licenses(const void *a, const void *b)
 static void
 flickcurl_read_licenses(flickcurl *fc)
 {
-  const char * parameters[5][2];
-  int count = 0;
   xmlDocPtr doc = NULL;
   xmlXPathContextPtr xpathCtx = NULL; 
   xmlXPathObjectPtr xpathObj = NULL;
@@ -70,9 +68,11 @@ flickcurl_read_licenses(flickcurl *fc)
   int i;
   int size;
   
-  parameters[count][0]  = NULL;
+  flickcurl_init_params(fc);
 
-  if(flickcurl_prepare(fc, "flickr.photos.licenses.getInfo", parameters, count))
+  flickcurl_end_params(fc);
+
+  if(flickcurl_prepare(fc, "flickr.photos.licenses.getInfo"))
     goto tidy;
 
   doc = flickcurl_invoke(fc);
@@ -113,11 +113,12 @@ flickcurl_read_licenses(flickcurl *fc)
     l = (flickcurl_license*)calloc(sizeof(flickcurl_license), 1);
 
     for(attr = node->properties; attr; attr = attr->next) {
+      size_t attr_len = strlen((const char*)attr->children->content);
       const char *attr_name = (const char*)attr->name;
       char *attr_value;
       
-      attr_value = (char*)malloc(strlen((const char*)attr->children->content)+1);
-      strcpy(attr_value, (const char*)attr->children->content);
+      attr_value = (char*)malloc(attr_len + 1);
+      memcpy(attr_value, attr->children->content, attr_len + 1);
       
       if(!strcmp(attr_name, "id")) {
         l->id = atoi(attr_value);
@@ -129,7 +130,8 @@ flickcurl_read_licenses(flickcurl *fc)
           l->url = attr_value;
         else
           free(attr_value);
-      }
+      } else
+          free(attr_value);
     }
     
 #if FLICKCURL_DEBUG > 1
@@ -219,25 +221,22 @@ int
 flickcurl_photos_licenses_setLicense(flickcurl* fc, const char* photo_id,
                                      int license_id)
 {
-  const char* parameters[9][2];
-  int count = 0;
   xmlDocPtr doc = NULL;
   int result = 1;
   char license_id_s[5];
   
+  flickcurl_init_params(fc);
+
   if(!photo_id)
     return 1;
 
-  parameters[count][0]  = "photo_id";
-  parameters[count++][1]= photo_id;
-  parameters[count][0]  = "license_id";
+  flickcurl_add_param(fc, "photo_id", photo_id);
   sprintf(license_id_s, "%d", license_id);
-  parameters[count++][1]= license_id_s;
+  flickcurl_add_param(fc, "license_id", license_id_s);
 
-  parameters[count][0]  = NULL;
+  flickcurl_end_params(fc);
 
-  if(flickcurl_prepare(fc, "flickr.photos.licenses.setLicense", parameters,
-                       count))
+  if(flickcurl_prepare(fc, "flickr.photos.licenses.setLicense"))
     goto tidy;
 
   flickcurl_set_write(fc, 1);
