@@ -2,7 +2,7 @@
  *
  * flickr.photos.upload-api.c - Flickr flickr.photos.upload.* API calls
  *
- * Copyright (C) 2007-2008, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2007-2012, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -57,23 +57,22 @@ flickcurl_ticket**
 flickcurl_photos_upload_checkTickets(flickcurl* fc,
                                      const char** tickets_ids)
 {
-  const char* parameters[8][2];
-  int count = 0;
   xmlDocPtr doc = NULL;
   xmlXPathContextPtr xpathCtx = NULL; 
   flickcurl_ticket** tickets = NULL;
   char* tickets_ids_string = NULL;
   
+  flickcurl_init_params(fc);
+
   if(!tickets_ids)
     return NULL;
 
   tickets_ids_string = flickcurl_array_join(tickets_ids, ',');
-  parameters[count][0]  = "tickets";
-  parameters[count++][1]= tickets_ids_string;
+  flickcurl_add_param(fc, "tickets", tickets_ids_string);
 
-  parameters[count][0]  = NULL;
+  flickcurl_end_params(fc);
 
-  if(flickcurl_prepare(fc, "flickr.photos.upload.checkTickets", parameters, count))
+  if(flickcurl_prepare(fc, "flickr.photos.upload.checkTickets"))
     goto tidy;
 
   doc = flickcurl_invoke(fc);
