@@ -119,11 +119,12 @@ flickcurl_build_tags(flickcurl* fc, flickcurl_photo* photo,
     t->photo = photo;
     
     for(attr = node->properties; attr; attr = attr->next) {
+      size_t attr_len = strlen((const char*)attr->children->content);
       const char *attr_name = (const char*)attr->name;
       char *attr_value;
 
-      attr_value = (char*)malloc(strlen((const char*)attr->children->content)+1);
-      strcpy(attr_value, (const char*)attr->children->content);
+      attr_value = (char*)malloc(attr_len + 1);
+      memcpy(attr_value, attr->children->content, attr_len + 1);
       
       if(!strcmp(attr_name, "id"))
         t->id = attr_value;
@@ -149,7 +150,8 @@ flickcurl_build_tags(flickcurl* fc, flickcurl_photo* photo,
         /* from tags.getHotList <tag score = "NN">TAG</tag> */
         t->count = atoi(attr_value);
         free(attr_value);
-      }
+      } else
+        free(attr_value);
     }
 
     /* Walk children nodes for <raw> element or text */
@@ -157,13 +159,15 @@ flickcurl_build_tags(flickcurl* fc, flickcurl_photo* photo,
       const char *chnode_name = (const char*)chnode->name;
       if(chnode->type == XML_ELEMENT_NODE) {
         if(saw_clean && !strcmp(chnode_name, "raw")) {
-          t->raw = (char*)malloc(strlen((const char*)chnode->children->content)+1);
-          strcpy(t->raw, (const char*)chnode->children->content);
+          size_t len = strlen((const char*)chnode->children->content);
+          t->raw = (char*)malloc(len + 1);
+          memcpy(t->raw, chnode->children->content, len + 1);
         }
       } else if(chnode->type == XML_TEXT_NODE) {
         if(!saw_clean) {
-          t->cooked = (char*)malloc(strlen((const char*)chnode->content)+1);
-          strcpy(t->cooked, (const char*)chnode->content);
+          size_t len = strlen((const char*)chnode->children->content);
+          t->cooked = (char*)malloc(len + 1);
+          memcpy(t->cooked, chnode->children->content, len + 1);
         }
       }
     }
@@ -220,8 +224,8 @@ flickcurl_build_tags_from_string(flickcurl* fc, flickcurl_photo* photo,
     
     len = p-string;
 
-    t->cooked = (char*)malloc(len+1);
-    strncpy(t->cooked, string, len);
+    t->cooked = (char*)malloc(len + 1);
+    memcpy(t->cooked, string, len);
     t->cooked[len] = '\0';
     
     if(fc->tag_handler)
@@ -338,8 +342,10 @@ flickcurl_build_tag_clusters(flickcurl* fc,
         tags_count = atoi(attr_value);
     }
 
-    if(tags_count <= 0)
+    if(tags_count <= 0) {
+      free(tc);
       continue;
+    }
     
     tc->tags = (char**)calloc(sizeof(char*), tags_count+1);
 
@@ -348,8 +354,9 @@ flickcurl_build_tag_clusters(flickcurl* fc,
       const char *chnode_name = (const char*)chnode->name;
       if(chnode->type == XML_ELEMENT_NODE && !strcmp(chnode_name, "tag")) {
         size_t len = strlen((const char*)chnode->children->content);
-        char *tag_name = (char*)malloc(len+1);
-        strcpy(tag_name, (const char*)chnode->children->content);
+        char *tag_name = (char*)malloc(len + 1);
+
+        memcpy(tag_name, (const char*)chnode->children->content, len + 1);
         tc->tags[tc->count++] = tag_name;
 
 #if FLICKCURL_DEBUG > 1
@@ -367,6 +374,12 @@ flickcurl_build_tag_clusters(flickcurl* fc,
   tcs->clusters[tcs->count] = NULL;
 
  tidy:
+  if(fc->failed) {
+    if(tcs)
+      flickcurl_free_tag_clusters(tcs);
+    tcs = NULL;
+  }
+
   if(xpathObj)
     xmlXPathFreeObject(xpathObj);
 
